/* HUB ISP - Program ATMEGA328P using only a USB HUB and simple circuitry
 * Copyright 2010, Paul Stoffregen <paul@pjrc.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "headers.h"


static usb_dev_handle *hub=NULL;


// Open a USB hub or device
//
void open_usb(int vid, int pid)
{
	struct usb_bus *bus;
	struct usb_device *dev;
	char name[32];
	int port;

	usb_init();
	usb_find_busses();
	usb_find_devices();
	for (bus = usb_get_busses(); bus; bus = bus->next) {
		for (dev = bus->devices; dev; dev = dev->next) {
			if (dev->descriptor.idVendor != vid) continue;
			if (dev->descriptor.idProduct != pid) continue;
			hub = usb_open(dev);
			if (!hub) die("unable to open HUB device\n");
			if (usb_get_driver_np(hub, 0, name, sizeof(name)) >= 0) {
				printf("HUB is in use by kernel driver \"%s\"\n", name);
				if (usb_detach_kernel_driver_np(hub, 0) < 0) {
					die("unable to detach kernel driver\n");
				}
				printf("Detach kernel driver successfully :)\n");
			}
			for (port=1; port <= 4; port++) {
				if (usb_control_msg(hub, 0x23, 3, 2, port, NULL, 0, 100) != 0) {
					die("error sending port suspend, port %d\n", port);
				}
			}
			return;
		}
	}
	die("Unable to find HUB with VID:PID = %04x:%04x\n", vid, pid);
}

// Close a USB hub or device
//
void close_usb(void)
{
	if (hub) {
		usb_release_interface(hub, 0);
		usb_close(hub);
		hub = NULL;
	}
}

// Read the voltage on the green wire of a port.
//
int read_port(int port)
{
	char buf[4];

	// get port status
	if (usb_control_msg(hub, 0xA3, 0, 0, port, buf, 4, 100) != 4) {
		die("error reading status from port %d\n", port);
	}
	//printf("%02X %02X\n", buf[0], buf[1]);
	// usb 2.0 spec, table 11-21, page 427
	// bit 0 = connect (either signal has a pullup)
	// bit 9 = low speed (pullup is on white white if set)
	if ((buf[0] & 0x01) && !(buf[1] & 0x02)) return 1;
	return 0;
}

// Generate an active-low pulse on the green wire of a port
//
void pulse_begin_port(int port)
{
	// reset port - pulls both lines low for approx 10 ms
	if (usb_control_msg(hub, 0x23, 3, 4, port, NULL, 0, 100) != 0) {
		die("error sending port reset, port %d\n", port);
	}
	// immediately suspend the port, before reset ends!
	if (usb_control_msg(hub, 0x23, 3, 2, port, NULL, 0, 100) != 0) {
		die("error sending port suspend, port %d\n", port);
	}
}

// Wait for the pulse to end
//
void pulse_end_port(int port)
{
	char buf[4];

	// wait for the reset condition to end
	do {
		if (usb_control_msg(hub, 0xA3, 0, 0, port, buf, 4, 100) != 4) {
			die("error reading status from port %d\n", port);
		}
		//printf("%02X\n", buf[0]);
	} while (buf[0] & 0x10);
}


// Generate an active-low pulse on the green wire of a port
//
void pulse_port(int port)
{
	pulse_begin_port(port);
	pulse_end_port(port);
}

